@@ -19,11 +19,16 @@ else:
     _MAX_INT = sys.maxint
 
 DATA_TYPE_OPTION = {
+    "common": ["ncalls", "choices"],
     "int": ["min_num", "max_num"],
     "float": ["min_num", "max_num", "positive"],
     "str": ["encoding", "fixed_length", "max_length", "str_attrs"],
+    "*_of": ["items", "min_items", "max_items"],
 }
 
+DATA_TYPE_OPTIONS = set()
+DATA_TYPE_OPTIONS.update(*DATA_TYPE_OPTION.values())
+
 _MIN_FLOAT = -1e7
 _MAX_FLOAT = 1e7
 _MIN_FLOAT_MAG = 1e-7
@@ -79,8 +84,6 @@ def get_float(min_num=_MIN_FLOAT, max_num=_MAX_FLOAT,
 def get_str(encoding=None, fixed_length=None, max_length=32, str_attrs=None):
     base, end = str_attrs if str_attrs else (_ASCII, _ASCII_LEN)
     length = fixed_length if fixed_length else random.randint(0, max_length)
-    if end < length:
-        length = end
     s = "".join(getitem(base, random.randint(0, end)) for _ in range(length))
     if encoding and not IS_PY3:
         s = unicode(s, encoding)
@@ -139,10 +142,73 @@ def generate(data, **kwargs):
         for key, value in data.items():
             _dict[key] = retrieve_func(value)(generate(value, **kwargs))
         yield _dict
+    elif isinstance(data, Generator):
+        yield data.generate(**kwargs)
     elif data is None:
         yield None
     else:
-        raise NotImplementedError("Unknown data type")
+        raise NotImplementedError("Unknown data type: %s" % data)
+
+class Generator(object):
+
+    def __init__(self, **options):
+        self.options = options
+
+    def generate(self, **kwargs):
+        raise NotImplementedError()
+
+    @staticmethod
+    def generate_data(data, **kwargs):
+        data_type, retrieve = parse(data)
+        return retrieve(generate(data_type, **kwargs))
+
+class list_of(Generator):
+    def __init__(self, data, **kwargs):
+        self.data = data
+        super(list_of, self).__init__(**kwargs)
+
+    def generate(self, **kwargs):
+        k = _options_to_num_items(self.options)
+
+        return [self.generate_data(self.data, **kwargs) for _ in range(k)]
+
+def _options_to_num_items(options):
+        options = dict(options)
+        min_items = options.pop("min_items", 0)
+        max_items = options.pop("max_items", 20)
+        if "items" in options:
+            min_items = max_items = options.pop("items")
+        assert min_items >= 0
+        assert max_items >= min_items
+        if options:
+            raise NotImplementedError(
+                "generator does not take an option '%s', "
+                "try giving it as a keyword argument to "
+                "the randomize() function" %
+                next(iter(options)))
+        return get_int(min_items, max_items)
+
+def nonempty_list_of(data, **options):
+    options.setdefault("min_items", 1)
+    return list_of(data, **options)
+
+class dict_of(Generator):
+    def __init__(self, keys, values, **kwargs):
+        self.keys = keys
+        self.values = values
+        super(dict_of, self).__init__(**kwargs)
+
+    def generate(self, **kwargs):
+        k = _options_to_num_items(self.options)
+
+        def gen(data, n):
+            return (self.generate_data(data, **kwargs) for _ in range(n))
+
+        # keys need to be unique
+        keys = set()
+        while len(keys) < k:
+            keys.update(gen(self.keys, k - len(keys)))
+        return dict(zip(keys, gen(self.values, k)))
 
 def retrieve_func(data):
     if isinstance(data, (list, set, tuple)):
